# Password Breach Alert

Password Breach Alert (PBA) is a Rails gem built as a Devise extension. PBA has two purposes:

- Signup password check: Avoid users register with a weak, common or compromised password
- Periodic breaches check: Check if users emails pop up in a recent verified security breach, and act accordingly

## Installation

In your Gemfile:

```
gem 'password_breach_alert'
```

Or, from the command line:

```
gem install password_breach_alert
```

## Inspirations and thanks

- [';--have i been pwned?][haveibeenpwned]: The start of it all. Big thanks to Troy Hunt.
- [zxcvbn]: For me, it was love at first sight.
- [Devise]: The de-facto standard for authentication in Rails.
- [devise-uncommon_password], [devise_zxcvbn] and [devise-pwned_password]: I integrated these three gems directly into this one. I learned a lot looking into the source code to merge them in one.
- [Signal v. Noise breach blog post][svn breach]: Big thanks to Basecamp for sharing this story, and getting things rolling for me.

### Signup password check

To enable the check of the password during registration, in your devisable model (generally, `User`), add `devise :password_breach_alert`

```ruby
# user.rb

class User < ApplicationRecord
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :validatable, :password_breach_alert
end
```

### Periodic breaches check

To check for breaches, and act with default behaviour, use `rails password_breach_alert:checker`. To periodically check for new breaches, put this rake task in your `cron` routine. For instance, if you use `whenever`, add something like the following snippet in your `schedule.rb`:

```ruby
# schedule.rb

every 1.day, at: '8:30 am' do
  rake 'password_breach_alert:checker'
end
```

## Configuration

You can configure varous aspects of both the Signup password check and the Periodic breaches check, using `devise` config options structure.

| config key                  | default value          | meaning                                                         |
| --------------------------- | ---------------------- | --------------------------------------------------------------- |
| pwned_active                | true                   | Does it check the password against HIBP endpoint?               |
| common_active               | true                   | Does it check the password against a list of common password?   |
| zxcvbn_active               | true                   | Does it check the password with zxcvbn?                         |
| breaches_policy             | SendDeviseNotification | What policy gets applies if some breaches are found for a user? |
| breaches_filter             | NewWithUser            | How does it decide when a breach is applicable to a user?       |
| breaches_filter_options     | -> { {} }              | The options passed to breaches_filter class                     |
| min_password_matches        | 1                      | pwned config                                                    |
| min_password_matches_warn   | nil                    | pwned config                                                    |
| pwned_password_open_timeout | 5                      | pwned config                                                    |
| pwned_password_read_timeout | 5                      | pwned config                                                    |
| zxcvbn_min_password_score   | 4                      | zxcvbn config                                                   |
| zxcvbn_tester               | Tester.new             | zxcvbn config                                                   |
| password_matches            | 100                    | common config                                                   |

### Signup password check (General)

- `pwned_active`: If `true` (default `true`) validates the password using HIBP apis
- `common_active`: If `true` (default `true`) validates the password against a list of common password
- `zxcvbn_active`: If `true` (default `true`) validates the password using zxcvbn

### Signup password check (pwned)

- `min_password_matches`: see [devise-pwned_password gem][devise-pwned_password]
- `min_password_matches_warn`: see [devise-pwned_password gem][devise-pwned_password]
- `pwned_password_open_timeout`: see [devise-pwned_password gem][devise-pwned_password]
- `pwned_password_read_timeout`: see [devise-pwned_password gem][devise-pwned_password]

### Signup password check (zxcvbn)

- `zxcvbn_min_password_score`: see [devise_zxcvbn gem][devise_zxcvbn]
- `zxcvbn_tester`: see [devise_zxcvbn gem][devise_zxcvbn]

### Signup password check (common)

- `password_matches`: see [devise-uncommon_password gem][devise-uncommon_password]

### Periodic breaches check

- `breaches_policy`: What policy gets applies if some breaches are found for a user? Default is `SendDeviseNotification`, that sends an email to the user
- `breaches_filter`: How does it decide when a breach is applicable to a user? Default is `'PasswordBreachAlert::BreachesFilters::NewWithUser'`, that filters all new breaches where the user email is involved. Other possibilities are `'PasswordBreachAlert::BreachesFilters::AllWithUser'`, that filters all breaches new or old where the user email is involved; or `'PasswordBreachAlert::BreachesFilters::AfterUserLastCheckedAt'`, that returns the breaches with `breach_date` later then checked, then updates the date on user, and requires the field `password_breach_alert_checked_at` on `User` table.
- `breaches_filter_options`: Some breaches filters require or permit additional options when instantiated. `AfterUserLastCheckedAt` gets called with default options `api: PasswordBreachAlert::Api::Breachedaccount.new, field: :password_breach_alert_checked_at`. `NewWithUser` and `AllWithUser` get called with default `api: PasswordBreachAlert::Api::Breachedaccount.new`.

## Motivation

The safety of an app relies on two factors: system and users.

Developers and Companies can mess up the system, and can mess up hard. BUT, following all the best practices is not so hard, and Rails developers have a good time with [Devise].

Reality is, the real weak link is always the user.

For instance, if a user uses the same email-password combination on different applications, and one of them gets breached, all the accesses have to be considered unsafe.

How can you mitigate this risk? At least in two different ways:

- When registering, block the sign up process if they use unsafe or weak or common passwords.
- When a new breach gets known, check if there are users with email present in the breached database on your system. For each of them, you can implement different strategies:
  - send an email to the owner, warning him
  - expire all session, block the logins, send a password reset email.
  - implement a temporary additional check required for the login (ie. captcha)
  - until the password is changed, allow the login only from known IP addresses.

## License

The gem is available as open source under the terms of the [MIT License](https://opensource.org/licenses/MIT).

## Work with us

This gem was developed as a 64hours side project (8 hours for 8 weeks) @[Uqido]. More info in a series of blog posts, starting with [the introduction (ITA)][password-breach-alert-introduzione].

We are always [looking for talented developers][work with us].

[svn breach]: https://m.signalvnoise.com/protecting-basecamp-from-breached-passwords/
[devise]: https://github.com/plataformatec/devise 'Flexible authentication solution for Rails with Warden.'
[devise-pwned_password]: https://github.com/michaelbanfield/devise-pwned_password
[devise-uncommon_password]: https://github.com/HCLarsen/devise-uncommon_password
[devise_zxcvbn]: https://github.com/bitzesty/devise_zxcvbn
[haveibeenpwned]: https://haveibeenpwned.com/ 'Check if you have an account that has been compromised in a data breach'
[zxcvbn]: https://github.com/dropbox/zxcvbn 'Low-Budget Password Strength Estimation'
[uqido]: https://www.uqido.com
[password-breach-alert-introduzione]: https://tech.uqido.com/2019/06/05/password-breach-alert-introduzione/
[work with us]: https://www.uqido.com/en/careers-en/
